#
# Init is pretty simple, we only patch RAMCacheManager CLASS by
# decorating get and set methods.
# This is done the safe way : this product can be refreshed without
# breaking anything.
#

import OFS.Cache
from zLOG import LOG, INFO, WARNING

PKG = 'RAMCacheForAnonymous'


def ZCache_get_decorator(self, ob, view_name='', keywords=None,
                   mtime_func=None, default=None, **kw):
    if hasattr(ob, 'REQUEST') and str(ob.REQUEST.get('AUTHENTICATED_USER', 'Anonymous User')) != 'Anonymous User':
        # just return default to disable caching for authenticated users
        #LOG('cache_deco', WARNING, "AUTHENTICATED CACHING : " + str(ob.REQUEST.get('AUTHENTICATED_USER')))
        return default
    #LOG('cache_deco', WARNING, "CALLING DECORATED METHOD")
    #  allow to return
    return self.__old_ZCache_get(ob, view_name, keywords,
                                 mtime_func, default, **kw)

def ZCache_set_decorator(self, ob, data, view_name='', keywords=None, mtime_func=None):
    if hasattr(ob, 'REQUEST') and str(ob.REQUEST.get('AUTHENTICATED_USER', 'Anonymous User')) != 'Anonymous User':
        # no cache set for authenticated users => save memory that would not be used
        return
    return self.__old_ZCache_set(ob, data, view_name, 
			         keywords, mtime_func)

# this function may be used with Caches other than RAMCache
def patch_cache(cache_class):
    if OFS.Cache.Cache in cache_class.__bases__:
         # flag to avoid overwriting original methods if this product is refreshed
        flag = 'monkey_patched_by_' + PKG
        if not hasattr(cache_class, flag):
            setattr(cache_class, flag, True)
            cache_class.__old_ZCache_get = cache_class.ZCache_get
            cache_class.__old_ZCache_set = cache_class.ZCache_set
        LOG(PKG, INFO, "Monkey Patching " + getattr(cache_class, '__file__', str(cache_class)))
        cache_class.ZCache_get = ZCache_get_decorator
        cache_class.ZCache_set = ZCache_set_decorator
    else:
        LOG(PKG, WARNING, "patch_cache:" + str(cache) + " is not a subclass of OFS.Cache.Cache => NOT PATCHING")



def initialize(context):
    """initialize the product by doing the patching"""
    
    from Products.StandardCacheManagers.RAMCacheManager import RAMCache
    patch_cache(RAMCache)


